'use strict';

var bundleTask = require('../../../tasks/javascript/bundle');
var expect = require('chai').expect;

describe('Task : javascript bundle', function () {

    var tempDir = 'test/.tmp';
    var fs = require('fs-extra');
    var path = require('path');

    function getTestClass() {
        return 'export default class TestClass {\n' +
            '\tconstructor (data) {\n' +
            '\t\tthis.property = data;\n' +
            '\t}\n' +
            '}\n\n' +
            'export var __useDefault = true;';
    }

    function bundle (bundleOptions) {

        var sources = ['TestClass.js'];

        var options = {
            destination: path.join(tempDir, 'transpiled'),
            configLocation: 'test/tasks/javascript/system.conf.js',
            base: tempDir
        };

        if (bundleOptions) {
            options.bundles = bundleOptions;
        }

        return Promise.all(sources.map(bundleTask(options)));
    }

    beforeEach(function () {
        fs.mkdir(tempDir);
        fs.writeFile(path.join(tempDir, 'TestClass.js'), getTestClass());
    });

    afterEach(function () {
        fs.removeSync(tempDir);
    });

    it('should create bundle if bundle config not present', function (done) {
        return bundle().then(function () {
            expect(fs.existsSync(path.join(tempDir, 'transpiled', 'TestClass.js'))).to.equal(true);
            done();
        });
    });

    it('should create sfx bundle that can be loaded without SystemJS', function (done) {
        return bundle({
            'TestClass.js': {
                name: 'TestClass',
                sfx: true
            }
        }).then(function () {
            expect(fs.existsSync(path.join(tempDir, 'transpiled', 'testClass.js'))).to.equal(true);

            var TestClass = require(path.join(process.cwd(), tempDir, 'transpiled', 'TestClass.js'));
            var instance = new TestClass.default('testProperty'); // eslint-disable-line new-cap
            expect(instance.property).to.equal('testProperty');
            done();
        });
    });

    it('should create bundle that has to be loaded with SystemJS', function (done) {
        return bundle({
            'TestClass.js': {
                sfx: false
            }
        }).then(function () {
            expect(fs.existsSync(path.join(tempDir, 'transpiled', 'testClass.js'))).to.equal(true);

            var TestClass = require(path.join(process.cwd(), tempDir, 'transpiled', 'TestClass.js'));
            var instance = new TestClass.default('testProperty'); // eslint-disable-line new-cap
            expect(instance.property).to.equal('testProperty');
            done();
        });
    });

    it('should create sfx bundles of formats specified', function (done) {
        return bundle({
            'TestClass.js': {
                name: 'TestClass',
                sfx: true,
                format : ['amd', 'cjs']
            }
        }).then(function () {
            expect(fs.existsSync(path.join(tempDir, 'transpiled', 'amd', 'testClass.js'))).to.equal(true);
            expect(fs.existsSync(path.join(tempDir, 'transpiled', 'cjs', 'testClass.js'))).to.equal(true);
            done();
        });
    });
});
